@@ -23,6 +23,7 @@ LICENSE file or <http://www.boost.org/LICENSE_1_0.txt>
 #endif
 
 #define CellObject_Check(o) PyObject_TypeCheck((o), &cell_object_type)
+#define RaithDataObject_Check(o) PyObject_TypeCheck((o), &raithdata_object_type)
 #define FlexPathObject_Check(o) PyObject_TypeCheck((o), &flexpath_object_type)
 #define LabelObject_Check(o) PyObject_TypeCheck((o), &label_object_type)
 #define LibraryObject_Check(o) PyObject_TypeCheck((o), &library_object_type)
@@ -50,6 +51,9 @@ static int return_error(ErrorCode error_code) {
         case ErrorCode::NoError:
             return 0;
         // Warnings
+        case ErrorCode::EmptyPath:
+            if (PyErr_WarnEx(PyExc_RuntimeWarning, "Empty path.", 1) != 0) return -1;
+            return 0;
         case ErrorCode::MissingReference:
             if (PyErr_WarnEx(PyExc_RuntimeWarning, "Missing reference.", 1) != 0) return -1;
             return 0;
@@ -161,6 +165,11 @@ struct RepetitionObject {
     Repetition repetition;
 };
 
+struct RaithDataObject {
+    PyObject_HEAD;
+    RaithData raith_data;
+};
+
 static PyTypeObject curve_object_type = {PyVarObject_HEAD_INIT(NULL, 0) "gdstk.Curve",
                                          sizeof(CurveObject),
                                          0,
@@ -441,6 +450,46 @@ static PyTypeObject repetition_object_type = {PyVarObject_HEAD_INIT(NULL, 0) "gd
                                               0,
                                               0};
 
+static PyTypeObject raithdata_object_type = {PyVarObject_HEAD_INIT(NULL, 0) "gdstk.RaithData",
+                                             sizeof(RaithDataObject),
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
+                                             raithdata_object_type_doc,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             0,
+                                             PyType_GenericNew,
+                                             0,
+                                             0};
+
 static PyTypeObject cell_object_type = {PyVarObject_HEAD_INIT(NULL, 0) "gdstk.Cell",
                                         sizeof(CellObject),
                                         0,
@@ -746,6 +795,7 @@ static Array<Vec2> custom_bend_function(double radius, double initial_angle, dou
 #include "label_object.cpp"
 #include "library_object.cpp"
 #include "polygon_object.cpp"
+#include "raithdata_object.cpp"
 #include "rawcell_object.cpp"
 #include "reference_object.cpp"
 #include "repetition_object.cpp"
@@ -1871,6 +1921,12 @@ static int gdstk_exec(PyObject* module) {
     reference_object_type.tp_getset = reference_object_getset;
     reference_object_type.tp_str = (reprfunc)reference_object_str;
 
+    raithdata_object_type.tp_dealloc = (destructor)raithdata_object_dealloc;
+    raithdata_object_type.tp_init = (initproc)raithdata_object_init;
+    // raithdata_object_type.tp_methods = raithdata_object_methods;
+    raithdata_object_type.tp_getset = raithdata_object_getset;
+    raithdata_object_type.tp_str = (reprfunc)raithdata_object_str;
+
     flexpath_object_type.tp_dealloc = (destructor)flexpath_object_dealloc;
     flexpath_object_type.tp_init = (initproc)flexpath_object_init;
     flexpath_object_type.tp_methods = flexpath_object_methods;
@@ -1920,16 +1976,14 @@ static int gdstk_exec(PyObject* module) {
     repetition_object_type.tp_getset = repetition_object_getset;
     repetition_object_type.tp_str = (reprfunc)repetition_object_str;
 
-    char const* names[] = {
-        "Library",   "Cell",       "Polygon", "FlexPath", "RobustPath", "Label",
-        "Reference", "Repetition", "Curve",   "RawCell",  "GdsWriter",
-    };
+    char const* names[] = {"Library",    "Cell",       "Polygon", "RaithData",
+                           "FlexPath",   "RobustPath", "Label",   "Reference",
+                           "Repetition", "Curve",      "RawCell", "GdsWriter"};
     PyTypeObject* types[] = {
-        &library_object_type,   &cell_object_type,       &polygon_object_type,
-        &flexpath_object_type,  &robustpath_object_type, &label_object_type,
-        &reference_object_type, &repetition_object_type, &curve_object_type,
-        &rawcell_object_type,   &gdswriter_object_type,
-    };
+        &library_object_type,   &cell_object_type,      &polygon_object_type,
+        &raithdata_object_type, &flexpath_object_type,  &robustpath_object_type,
+        &label_object_type,     &reference_object_type, &repetition_object_type,
+        &curve_object_type,     &rawcell_object_type,   &gdswriter_object_type};
     for (unsigned long i = 0; i < sizeof(types) / sizeof(types[0]); ++i) {
         if (PyType_Ready(types[i]) < 0) {
             Py_DECREF(module);
@@ -1949,7 +2003,7 @@ static int gdstk_exec(PyObject* module) {
 static PyModuleDef_Slot gdstk_slots[] = {{Py_mod_exec, (void*)gdstk_exec}, {0, NULL}};
 
 static struct PyModuleDef gdstk_module = {PyModuleDef_HEAD_INIT,
-                                          "gdstk",
+                                          "_gdstk",
                                           gdstk_module_doc,
                                           0,
                                           gdstk_methods,
@@ -1958,7 +2012,7 @@ static struct PyModuleDef gdstk_module = {PyModuleDef_HEAD_INIT,
                                           NULL,
                                           NULL};
 
-PyMODINIT_FUNC PyInit_gdstk(void) {
+PyMODINIT_FUNC PyInit__gdstk(void) {
     PyDateTime_IMPORT;
     PyObject* module = PyModuleDef_Init(&gdstk_module);
     if (!module) {
