@@ -169,8 +169,9 @@ static PyObject* label_object_delete_property(LabelObject* self, PyObject* args)
 static PyObject* label_object_set_gds_property(LabelObject* self, PyObject* args) {
     uint16_t attribute;
     char* value;
-    if (!PyArg_ParseTuple(args, "Hs:set_gds_property", &attribute, &value)) return NULL;
-    set_gds_property(self->label->properties, attribute, value);
+    Py_ssize_t count;
+    if (!PyArg_ParseTuple(args, "Hs#:set_gds_property", &attribute, &value, &count)) return NULL;
+    if (count >= 0) set_gds_property(self->label->properties, attribute, value, (uint64_t)count);
     Py_INCREF(self);
     return (PyObject*)self;
 }
@@ -183,7 +184,13 @@ static PyObject* label_object_get_gds_property(LabelObject* self, PyObject* args
         Py_INCREF(Py_None);
         return Py_None;
     }
-    return PyUnicode_FromString((char*)value->bytes);
+    PyObject* result = PyUnicode_FromStringAndSize((char*)value->bytes, (Py_ssize_t)value->count);
+    if (PyErr_Occurred()) {
+        Py_XDECREF(result);
+        PyErr_Clear();
+        result = PyBytes_FromStringAndSize((char*)value->bytes, (Py_ssize_t)value->count);
+    }
+    return result;
 }
 
 static PyObject* label_object_delete_gds_property(LabelObject* self, PyObject* args) {
@@ -196,7 +203,8 @@ static PyObject* label_object_delete_gds_property(LabelObject* self, PyObject* a
 
 static PyMethodDef label_object_methods[] = {
     {"copy", (PyCFunction)label_object_copy, METH_NOARGS, label_object_copy_doc},
-    {"__deepcopy__", (PyCFunction)label_object_deepcopy, METH_VARARGS | METH_KEYWORDS, label_object_deepcopy_doc},
+    {"__deepcopy__", (PyCFunction)label_object_deepcopy, METH_VARARGS | METH_KEYWORDS,
+     label_object_deepcopy_doc},
     {"apply_repetition", (PyCFunction)label_object_apply_repetition, METH_NOARGS,
      label_object_apply_repetition_doc},
     {"set_property", (PyCFunction)label_object_set_property, METH_VARARGS, object_set_property_doc},
