@@ -5,7 +5,8 @@ Boost Software License - Version 1.0.  See the accompanying
 LICENSE file or <http://www.boost.org/LICENSE_1_0.txt>
 */
 
-#include "polygon.h"
+#define __STDC_FORMAT_MACROS 1
+#define _USE_MATH_DEFINES
 
 #include <float.h>
 #include <inttypes.h>
@@ -14,13 +15,14 @@ LICENSE file or <http://www.boost.org/LICENSE_1_0.txt>
 #include <stdio.h>
 #include <string.h>
 
-#include "allocator.h"
-#include "clipper_tools.h"
-#include "font.h"
-#include "repetition.h"
-#include "sort.h"
-#include "utils.h"
-#include "vec.h"
+#include <gdstk/allocator.hpp>
+#include <gdstk/clipper_tools.hpp>
+#include <gdstk/font.hpp>
+#include <gdstk/polygon.hpp>
+#include <gdstk/repetition.hpp>
+#include <gdstk/sort.hpp>
+#include <gdstk/utils.hpp>
+#include <gdstk/vec.hpp>
 
 namespace gdstk {
 
@@ -78,6 +80,24 @@ double Polygon::signed_area() const {
     return 0.5 * result;
 }
 
+double Polygon::perimeter() const {
+    if (point_array.count < 3) return 0;
+    double result = 0;
+    Vec2* p = point_array.items;
+    Vec2 v0 = *p++;
+
+    for (uint64_t num = point_array.count - 1; num > 0; num--) {
+        Vec2 v1 = *p++ - v0;
+        result += v1.length();
+        v0 += v1;
+    }
+    result += (point_array.items[0] - point_array.items[point_array.count-1]).length();
+    if (repetition.type != RepetitionType::None) result *= repetition.get_count();
+    return result;
+}
+
+
+
 // Based on algorithm 7 from: Kai Hormann, Alexander Agathos, “The point in
 // polygon problem for arbitrary polygons,” Computational Geometry, Volume 20,
 // Issue 3, 2001, Pages 131-144, ISSN 0925-7721.
@@ -332,51 +352,58 @@ void Polygon::fracture(uint64_t max_points, double precision, Array<Polygon*>& r
         subj->bounding_box(min, max);
 
         const uint64_t num_cuts = num_points / max_points;
-        const double frac = num_points / (num_cuts + 1.0);
-        Array<double> cuts = {};
-        cuts.ensure_slots(num_cuts);
-        cuts.count = num_cuts;
         bool x_axis;
         double* coords = (double*)allocate(sizeof(double) * num_points);
         if (max.x - min.x > max.y - min.y) {
-            double* x = coords;
-            double* px = x;
-            Vec2* pt = subj->point_array.items;
-            for (uint64_t j = 0; j < num_points; j++) (*px++) = (pt++)->x;
-            sort(x, num_points);
             x_axis = true;
-            px = cuts.items;
-            for (uint64_t j = 0; j < num_cuts; j++) (*px++) = x[(uint64_t)((j + 1.0) * frac + 0.5)];
-        } else {
-            double* y = coords;
-            double* py = y;
+            double* x = coords;
             Vec2* pt = subj->point_array.items;
-            for (uint64_t j = 0; j < num_points; j++) (*py++) = (pt++)->y;
-            sort(y, num_points);
+            for (uint64_t j = 0; j < num_points; j++) (*x++) = (pt++)->x;
+        } else {
             x_axis = false;
-            py = cuts.items;
-            for (uint64_t j = 0; j < num_cuts; j++) (*py++) = y[(uint64_t)((j + 1.0) * frac + 0.5)];
+            double* y = coords;
+            Vec2* pt = subj->point_array.items;
+            for (uint64_t j = 0; j < num_points; j++) (*y++) = (pt++)->y;
+        }
+        sort(coords, num_points);
+        Array<double> interior_coords = {0, 0, coords};
+        while (interior_coords.items[0] == coords[0]) ++interior_coords.items;
+        interior_coords.count = num_points - (interior_coords.items - coords);
+        while (interior_coords.count > 0 &&
+                interior_coords.items[interior_coords.count - 1] == coords[num_points - 1])
+            --interior_coords.count;
+
+        Array<double> cuts = {};
+        if (interior_coords.count == 0) {
+            cuts.append((coords[0] + coords[num_points - 1]) * 0.5);
+        } else if (interior_coords.count <= num_cuts) {
+            cuts.extend(interior_coords);
+        } else {
+            cuts.ensure_slots(num_cuts);
+            const double frac = interior_coords.count / (num_cuts + 1.0);
+            for (uint64_t j = 1; j <= num_cuts; j++)
+                cuts.append(interior_coords[(uint64_t)(j * frac)]);
         }
         free_allocation(coords);
 
         Array<Polygon*>* chopped =
             (Array<Polygon*>*)allocate_clear((cuts.count + 1) * sizeof(Array<Polygon*>));
         slice(*subj, cuts, x_axis, scaling, chopped);
-        cuts.clear();
 
         subj->point_array.clear();
         result.remove_unordered(i);
         free_allocation(subj);
 
         uint64_t total = 0;
-        for (uint64_t j = 0; j <= num_cuts; j++) total += chopped[j].count;
+        for (uint64_t j = 0; j <= cuts.count; j++) total += chopped[j].count;
         result.ensure_slots(total);
 
-        for (uint64_t j = 0; j <= num_cuts; j++) {
+        for (uint64_t j = 0; j <= cuts.count; j++) {
             result.extend(chopped[j]);
             chopped[j].clear();
         }
 
+        cuts.clear();
         free_allocation(chopped);
     }
 
@@ -421,9 +448,10 @@ ErrorCode Polygon::to_gds(FILE* out, double scaling) const {
 
     uint64_t total = point_array.count + 1;
     if (total > 8190) {
-        fputs(
-            "[GDSTK] Polygons with more than 8190 are not supported by the official GDSII specification. This GDSII file might not be compatible with all readers.\n",
-            stderr);
+        if (error_logger)
+            fputs(
+                "[GDSTK] Polygons with more than 8190 are not supported by the official GDSII specification. This GDSII file might not be compatible with all readers.\n",
+                error_logger);
         error_code = ErrorCode::UnofficialSpecification;
     }
     Array<int32_t> coords = {};
@@ -1133,7 +1161,7 @@ Polygon racetrack(const Vec2 center, double straight_length, double radius, doub
     Vec2* v1 = result.point_array.items;
     Vec2* v2 = result.point_array.items + num_points;
     for (uint64_t i = 0; i < num_points; i++) {
-        const double angle = initial_angle + i * M_PI / num_points;
+        const double angle = initial_angle + i * M_PI / (num_points - 1);
         const Vec2 rad_vec = {radius * cos(angle), radius * sin(angle)};
         *v1++ = c1 + rad_vec;
         *v2++ = c2 - rad_vec;
@@ -1148,7 +1176,7 @@ Polygon racetrack(const Vec2 center, double straight_length, double radius, doub
         *v2++ = c1 + Vec2{inner_radius * cos(initial_angle), inner_radius * sin(initial_angle)};
         v1 = v2 + num_points;
         for (uint64_t i = num_points; i > 0; i--) {
-            const double angle = initial_angle + (i - 1) * M_PI / num_points;
+            const double angle = initial_angle + (i - 1) * M_PI / (num_points - 1);
             const Vec2 rad_vec = {inner_radius * cos(angle), inner_radius * sin(angle)};
             *v1++ = c1 + rad_vec;
             *v2++ = c2 - rad_vec;
@@ -1598,7 +1626,8 @@ ErrorCode contour(const double* data, uint64_t rows, uint64_t cols, double level
             }
         }
         if (!found) {
-            fprintf(stderr, "[GDSTK] Unable to process polygon hole in contour.\n");
+            if (error_logger)
+                fprintf(error_logger, "[GDSTK] Unable to process polygon hole in contour.\n");
             error_code = ErrorCode::BooleanError;
             hole->clear();
             free_allocation(hole);
