@@ -5,7 +5,8 @@ Boost Software License - Version 1.0.  See the accompanying
 LICENSE file or <http://www.boost.org/LICENSE_1_0.txt>
 */
 
-#include "robustpath.h"
+#define __STDC_FORMAT_MACROS 1
+#define _USE_MATH_DEFINES
 
 #include <inttypes.h>
 #include <math.h>
@@ -13,9 +14,10 @@ LICENSE file or <http://www.boost.org/LICENSE_1_0.txt>
 #include <stdio.h>
 #include <string.h>
 
-#include "allocator.h"
-#include "curve.h"
-#include "utils.h"
+#include <gdstk/allocator.hpp>
+#include <gdstk/curve.hpp>
+#include <gdstk/robustpath.hpp>
+#include <gdstk/utils.hpp>
 
 namespace gdstk {
 
@@ -1069,10 +1071,11 @@ ErrorCode RobustPath::spine_intersection(const SubPath &sub0, const SubPath &sub
             du1 /= norm_v1;
         }
     }
-    fprintf(
-        stderr,
-        "[GDSTK] No intersection found in RobustPath spine construction around (%lg, %lg) and (%lg, %lg).\n",
-        p0.x, p0.y, p1.x, p1.y);
+    if (error_logger)
+        fprintf(
+            error_logger,
+            "[GDSTK] No intersection found in RobustPath spine construction around (%lg, %lg) and (%lg, %lg).\n",
+            p0.x, p0.y, p1.x, p1.y);
     return ErrorCode::IntersectionNotFound;
 }
 
@@ -1123,10 +1126,11 @@ ErrorCode RobustPath::center_intersection(const SubPath &sub0, const Interpolati
             du1 /= norm_v1;
         }
     }
-    fprintf(
-        stderr,
-        "[GDSTK] No intersection found in RobustPath center construction around (%lg, %lg) and (%lg, %lg).\n",
-        p0.x, p0.y, p1.x, p1.y);
+    if (error_logger)
+        fprintf(
+            error_logger,
+            "[GDSTK] No intersection found in RobustPath center construction around (%lg, %lg) and (%lg, %lg).\n",
+            p0.x, p0.y, p1.x, p1.y);
     return ErrorCode::IntersectionNotFound;
 }
 
@@ -1178,10 +1182,11 @@ ErrorCode RobustPath::left_intersection(const SubPath &sub0, const Interpolation
             du1 /= norm_v1;
         }
     }
-    fprintf(
-        stderr,
-        "[GDSTK] No intersection found in RobustPath left side construction around (%lg, %lg) and (%lg, %lg).\n",
-        p0.x, p0.y, p1.x, p1.y);
+    if (error_logger)
+        fprintf(
+            error_logger,
+            "[GDSTK] No intersection found in RobustPath left side construction around (%lg, %lg) and (%lg, %lg).\n",
+            p0.x, p0.y, p1.x, p1.y);
     return ErrorCode::IntersectionNotFound;
 }
 
@@ -1233,10 +1238,11 @@ ErrorCode RobustPath::right_intersection(const SubPath &sub0, const Interpolatio
             du1 /= norm_v1;
         }
     }
-    fprintf(
-        stderr,
-        "[GDSTK] No intersection found in RobustPath right side construction around (%lg, %lg) and (%lg, %lg).\n",
-        p0.x, p0.y, p1.x, p1.y);
+    if (error_logger)
+        fprintf(
+            error_logger,
+            "[GDSTK] No intersection found in RobustPath right side construction around (%lg, %lg) and (%lg, %lg).\n",
+            p0.x, p0.y, p1.x, p1.y);
     return ErrorCode::IntersectionNotFound;
 }
 
@@ -1440,15 +1446,14 @@ ErrorCode RobustPath::to_polygons(bool filter, Tag tag, Array<Polygon *> &result
         uint64_t num =
             left_side.count + initial_cap.point_array.count + final_cap.point_array.count - 2;
         right_side.ensure_slots(num);
-        Vec2 *dst = right_side.items + right_side.count;
+        Vec2 *dst = right_side.items + right_side.count - 1;
 
-        memcpy(dst, final_cap.point_array.items + 1,
-               sizeof(Vec2) * (final_cap.point_array.count - 2));
-        dst += final_cap.point_array.count - 2;
+        memcpy(dst, final_cap.point_array.items, sizeof(Vec2) * final_cap.point_array.count);
+        dst += final_cap.point_array.count;
         final_cap.clear();
 
-        Vec2 *src = left_side.items + left_side.count - 1;
-        for (uint64_t i = left_side.count; i > 0; i--) *dst++ = *src--;
+        Vec2 *src = left_side.items + left_side.count - 2;
+        for (uint64_t i = left_side.count - 1; i > 0; i--) *dst++ = *src--;
         left_side.clear();
 
         memcpy(dst, initial_cap.point_array.items, sizeof(Vec2) * initial_cap.point_array.count);
